<!doctype html>
<head>
   <title>suite-py login</title>
   <style type="text/css">
      div {
      font-family: avenirltstd-medium, Helvetica, Verdana, sans-serif;
      width: 100%;
      height: 95vh;
      display: flex;
      flex-flow: column;
      justify-content: center;
      }
      .text-container {
      margin: auto;
      text-align: center;
      }
      .background {
      background-image: url("https://www.prima.it/frontale/images/home/hero.svg");
      background-repeat: no-repeat;
      background-position: center;
      background-size: cover;
      }
   </style>
</head>
<body>
   <div class="background">
      <div class="text-container">
         <h2>suite-py okta login</h2>
         <h3>You have been successfully signed in to okta.<br>
            You can now close this browser.
         </h3>
      </div>
   </div>
</body>
